function ListController($scope, $rootScope, $timeout, ScreenService, templateManager, ZHPromise, eventEngine, Query, CrosstabService,
	$filter, maskEngine, ZhUtil, $window, ApplicationContext, ZhFeatures, eventAggregator, DecodeValueFieldService, MessageService) {

	// BEGIN: Initialize
	var self = this;
	self.i18n = ScreenService.i18n;
	var locals = ApplicationContext;

	var AVG = 'AVG';
	var SUM = 'SUM';
	var MAX = 'MAX';
	var MIN = 'MIN';
	var COUNT = 'COUNT';

	var widgetAccessorDescriptor = {
		get: function () {
			return $scope.widget || $scope._currentPopup;
		}
	};

	Object.defineProperty(self, "widget", widgetAccessorDescriptor);

	function extendBehavior() {
		Util.decorateFromConstructor(self, OrderByable);
		Util.decorateFromConstructor(self, Searchable);
		Util.extend(self, new ListTemplatable());
	}
	extendBehavior();

	var moment = $window.moment;
	var allowMultiSelect = null;
	$scope.filtered = [];

	this.getAllowMultiSelect = function getAllowMultiSelect() {
		return allowMultiSelect;
	};

	eventAggregator.publish("OnListStartUp", {
		widget: $scope.widget
	});
	// END: Initialize

	// Floating Control defaults
	var floatingControlDefaults = {
		pdfAction: false,
		xlsAction: false,
		csvAction: false,
		customizationAction: {
			filter: true,
			layout: false,
			view: false
		}
	};
	self.updateFloatingControlDefaults = Util.buildDebounceMethod(function(options) {
		self.widget.floatingControl = self.widget.floatingControl !== false && Util.extend(options, self.widget.floatingControl || {});
	}, 100);
	self.updateFloatingControlDefaults(floatingControlDefaults);

	if(!self.widget.originalOrder) {
		self.widget.originalOrder = Util.clone(self.widget.order);
	}


	// BEGIN: Search
	ZhFeatures.registerFeatureUsage({
		'controller': 'ListController',
		'widgetName': $scope.widget ? $scope.widget.name : ''
	});

	this.search = function(gridRow) {
		self.widget.searchCriteria = self.widget.searchCriteria || {};
		self.widget.searchList = self.widget.searchCriteria._ALL || self.widget.searchCriteria._ALL_MAP;
		var rowVisibility = true;
		if (Util.objectHasAnyValue(self.widget.searchCriteria)) {
			rowVisibility = self.checkRowVisibilityFromCriteria(gridRow, self.widget.searchCriteria);
		}
		if (rowVisibility && !$scope.widget.dataSource.getStorageStrategy()) {
			if (!$scope.widget.applyFilters) {
				Util.extend($scope.widget, new Filterable(ZhUtil, Query));
			}

			rowVisibility = $scope.widget.applyFilters(gridRow, $scope.widget.getDataSourceFilter());
		}
		gridRow.__notSelectable = !rowVisibility;
		return rowVisibility;
	};

	this._searchWithPaginationReload = Util.buildDebounceMethod(_searchWithPaginationReload, 1000);

	function _searchWithPaginationReload(widget, callback) {
		var promise;
		if (Util.isArray(self.widget.fields) && self.widget.fields.length) {
			promise = widget.reload(ZhUtil.getLikeAllFilter(self.widget, self.widget.searchCriteria || {}));
		} else {
			promise = ZHPromise.reject();
		}
		if (callback) {
			callback(promise);
		}
	}

	var _redrawElement = function() {
		if (self.widget.resizeGridColumns) {
			self.widget.resizeGridColumns(true);
		}
	};

	if (!self.isGantt(self.widget)) {
		$scope.$watchCollection(function() {
			return self.widget.searchCriteria;
		}, function(current, old) {
			if (!angular.equals(current || {}, old || {})) {
				if (self.widget.resetInfinityScroll) {
					self.widget.resetInfinityScroll();
				}
				searchFromSearchCriteria();
			}
		});
	}

	var searchFromSearchCriteria = Util.buildDebounceMethod(function() {
		if (!self.widget._noReloadFromSearchCriteria) {
			self.widget.searchList = self.widget.searchCriteria && self.widget.searchCriteria._ALL || undefined;
			updateGroupedData();
			if (self.widget.searchWithPagination && self.widget.mode !== ViewMode.INLINE_EDIT) {
				self._searchWithPaginationReload(self.widget, function(searchPromise) {
					searchPromise.then(function() {
						updateGroupedData();
						_redrawElement();
					}, function() {
						_redrawElement();
					});
				});
			} else {
				_redrawElement();
			}
		}
		self.widget._noReloadFromSearchCriteria = false;
	}, 200);
	// END: Search

	// BEGIN: Toggle
	this.deselectAll = function deselectAll(widget) {
		var event = widget.unSelectAll();
		if (!event || !event.isDefaultPrevented()) {
			widget.dataSource.clearCheckedRows();
			if (!$scope._currentPopup) {
				$rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
			}
		}
	};

	var uiDeselectAllDeregister = $rootScope.$on("ui.deselectAll", function(event, widget) {
		self.deselectAll(widget);
	});

	this.onLongTap = function(row) {
		if (row.__group) return;

		if (this.widget.allowSelectOnLongTap) {
			this.toggle(row);
		}
	};

	this.toggle = function(row) {
		var widget = self.widget;
		widget.selectedRow = row;

		var event;
		if (widget.beforeSelectRow && !row.__isSelected) {
			event = widget.beforeSelectRow();
		} else if (widget.beforeUnselectRow && row.__isSelected) {
			event = widget.beforeUnselectRow();
		}

		if (!event || !event.isDefaultPrevented()) {

			var index = getIndex(row, $scope);
			if (index === -1) {
				row.__isSelected = true;
				if (!$scope._currentPopup) $rootScope.$broadcast("ui.addToFooter", "showOnMultiSelect");
				widget.dataSource.removeUncheckedRows(row);
			} else {
				delete row.__isSelected;
				widget.dataSource.removeCheckedRows(row);
				widget.dataSource.uncheckedRows.push(row);
			}

			if (self.isTree(self.widget)) {
				self.handleDependentRows(row, widget);
			}

			if (!$scope._currentPopup && !widget.dataSource.checkedRows.length) {
				$rootScope.$broadcast("ui.removeFromFooter", "showOnMultiSelect");
			}

			widget.updateCheckedRows();

			if (widget.afterSelectRow && row.__isSelected) {
				widget.afterSelectRow();
			} else if (widget.afterUnselectRow && !row.__isSelected) {
				widget.afterUnselectRow();
			}

		}
	};

	$scope.widget.toggle = self.toggle;

	this.getVisibleCheckedRows = function() {
		return (self.widget.groupedData || self.widget.dataSource.data).filter(function(row) {
			return row.__isSelected;
		});
	};

	this._toggleAllData = function() {
		self.widget.dataSource.allToggled = true;
		self._toggleAll();
	};

	this.toggleAll = function() {
		if (self.getVisibleCheckedRows().length === 0 && !self.widget.dataSource.loadedAllData) {
			messageConfig = {
				"type": 'CONFIRMATION',
				"success": (self.widget.selectAllWithoutRequest) ? self._toggleAllData : self._loadAllDataAndToggle,
				"fail": self._toggleAll
			};
			MessageService.confirmMessageByCode('zeedhi#MORE_ROWS_DISPLAY', undefined, messageConfig);
		} else {
			self._toggleAll();
		}
	};

	this._toggleAll = function(useDatasourceData) {
		var checkedRows = self.getVisibleCheckedRows();
		var rows = (!useDatasourceData && self.widget.groupedData) || self.widget.dataSource.data;
		if (checkedRows.length > 0) {
			self.widget.unSelectAll(rows);
		} else {
			self.widget.selectAll(rows);
		}
		self.widget.updateCheckedRows();
		if (self.widget.focusElement) self.widget.focusElement();
	};

	this._loadAllDataAndToggle = function() {
		var dataSourceFilters = self.widget.getDataSourceFilter();
		if(self.widget.searchCriteria){
			dataSourceFilters = dataSourceFilters.concat(ZhUtil.getLikeAllFilter(self.widget, self.widget.searchCriteria));
		}

		self.widget.dataSource.setCurrentPage(1); // reset curret page to reload all data
		return self.widget.dataSource.load(dataSourceFilters, DataSource.LOAD_ALL).then(function() {
			self.widget.dataSource.setCurrentPage(Math.ceil(self.widget.dataSource.data.length / (self.widget.itemsPerPage || self.widget.dataSource.itemsPerPage))); // set current page to highest number
			self.widget.infinityScroll();
			self._toggleAll(true);
		});
	};
	// END: Toggle

	// BEGIN: Others functions
	$scope.openRow = function(row, widget) {
		if (row.__group) return;

		if ((widget.showCheckbox && widget.dataSource.getCheckedRows().length) || allowMultiSelect === widget || widget.allowMultiSelect) {
			self.toggle(row);
		} else {
			widget.setCurrentRow(row);
		}
		widget.dataSource.updateCheckedRows();
	};

	$scope.openRowInnerList = function(row, widget) {
		if (!Util.isUndefined(widget.currentRow) &&
			widget.currentRow.__index === widget.dataSource.data.indexOf(row)) {
			delete widget.currentRow;
		} else {
			$scope.openRow(row, widget);
		}
	};

	function getIndex(row, scope) {
		return self.widget.dataSource.checkedRows.indexOf(row);
	}

	$scope.scrollTo = function(evt) {
		$timeout(function() {
			var item = $(evt.target).closest('li');
			var itemTop = item.position().top;
			var itemBottom = itemTop + item[0].clientHeight;
			var scroller = item.closest("[zh-scroller]");
			var top = 0;
			if (itemTop < 0 || itemBottom > scroller.height()) {
				if (itemTop < 0) {
					top = scroller.scrollTop() + itemTop - 10;
				} else {
					top = scroller.scrollTop() + itemBottom - scroller.height() + 10;
				}
				scroller.animate({
					scrollTop: top
				}, 500);
			}
		}, 100, false);
	};

	var uiAllowMultiSelectDeregister = $rootScope.$on("ui.allowMultiSelect", function(event, widget) {
		if ($scope._currentPopup) {
			return;
		}
		allowMultiSelect = widget;
	});

	var uiDisableMultiSelectDeregister = $rootScope.$on("ui.disableMultiSelect", function() {
		if ($scope._currentPopup) {
			return;
		}
		allowMultiSelect = null;
	});

	this.getGroupHeaderText = function getGroupHeaderText(row) {
		var groupHeaderText = "";
		if (row) {
			var middleText = row.__groupLabel && row.__groupValue ? ": " : "";
			var groupLabel = row.__groupLabel || "";
			var groupValue = row.__groupValue || "";
			groupHeaderText = groupLabel + middleText + groupValue;
		}

		return groupHeaderText;
	};

	this.getGroupFooterTitle = function getGroupFooterTitle(row) {
		var groupFooterTitle = "";
		if (row) {
			if (row.__groupValue) {
				var middleText = row.__groupLabel && row.__groupValue ? ": " : "";
				var groupLabel = row.__groupLabel || "";
				var groupValue = row.__groupValue || "";
				groupFooterTitle = ScreenService.i18n("Total ") + "(" + groupLabel + middleText + groupValue + ")";
			} else {
				groupFooterTitle = ScreenService.i18n("TOTAL");
			}
		}

		return groupFooterTitle;
	};

	// END: Others functions

	// BEGIN: Conditional actions control

	var hasConditionalActions = false;
	var leftActions = [];
	var rightActionsGrid = [];
	var rightActionsList = [];
	var rightActions = [];
	var defaultActionsValue = function() {
		$scope.visibleLeftActions = leftActions.length;
		$scope.visibleRightActions = rightActions.length;
		$scope.leftActions = leftActions;
		$scope.rightActions = rightActions;
	};
	var buildGridActions = function() {
		hasConditionalActions = false;
		leftActions = $scope.widget.getActionsByVisibility('grid-left');
		rightActionsGrid = $scope.widget.getActionsByVisibility('grid-right');
		rightActionsList = $scope.widget.getActionsByVisibility('list-right');
		rightActions = rightActionsGrid.concat(rightActionsList);

		$scope.widget.actions.forEach(function(item) {
			if (item.condition) {
				hasConditionalActions = true;
			}
		});

		if (!hasConditionalActions) {
			defaultActionsValue();
		}
	};
	$scope.buildGridActions = buildGridActions;
	buildGridActions();

	this.widget.updateGridActions = function updateGridActions() {
		_updateGridActions();
		templateManager.updateTemplate();
	};

	function updateGridActionsOnRow(row) {
		row = row || {};
		ApplicationContext.registerInstance('row', row);

		row.__rightActions = filterActionsAccordingToCondition(rightActions);
		row.__leftActions = filterActionsAccordingToCondition(leftActions);
		$scope.visibleRightActions = Math.max($scope.visibleRightActions, row.__rightActions.length);
		$scope.visibleLeftActions = Math.max($scope.visibleLeftActions, row.__leftActions.length);

		ApplicationContext.removeInstance('row');
	}

	function _updateGridActions() {
		if (hasConditionalActions) {

			$scope.visibleLeftActions = 0;
			$scope.visibleRightActions = 0;

			if ($scope.widget && $scope.widget.dataSource && $scope.widget.dataSource.data) {
				$scope.widget.dataSource.data.forEach(updateGridActionsOnRow);
			}
		}
	}

	function filterActionsAccordingToCondition(actions) {
		return actions.filter(function(action) {
			return !action.condition || ZhUtil.evaluate("@" + action.condition + "@");
		});
	}

	// END: Conditional actions control

	// BEGIN: Order by

	this.changeOrder = function(column) {
		if (!!column.disableOrdering) return;

		self.widget.selectOrder = false;
		self.widget.columnOrder = true;
		if (self.widget.order == column.name && self.widget.reverse) {
			self.widget.order = Util.clone(self.widget.originalOrder);
			self.widget.reverse = false;
		} else {
			self.widget.reverse = (self.widget.order == column.name) ? !self.widget.reverse : false;
			self.widget.order = column.name;
		}

		publishAfterChangeOrderEvent();
		this.updateOrderBy();
		updateGroupedData();
	};

	this.buildOrder = function(widget) {
		return ['-__isSelected'];
	};

	this.toggleSelectOrder = function(widget) {
		if (angular.equals(widget.order, widget.selectField.order) || widget.columnOrder) {
			widget.columnOrder = false;
			widget.selectOrder = true;
			widget.order = this.buildOrder(widget);
			widget.reverse = false;
		} else {
			if (!widget.reverse) {
				widget.reverse = true;
			} else {
				widget.order = widget.selectField.order;
				widget.selectOrder = false;
			}
		}

		updateGroupedData();
	};

	this.updateOrderBy = function(widgetIsLazyLoad) {
		self.updateOrderByColumns(self.widget);
		var dataSourceFilters = (self.widget.dataSourceFilter || []).filter(function(filter) {
			return filter.operator != Query.ORDER;
		});

		var orderByFilter = ZhUtil.getOrderByFilter(self.getOrderByColumns());
		if (orderByFilter.length > 0 && orderByFilter[0].name.length > 0) {
			dataSourceFilters = dataSourceFilters.concat(orderByFilter);
		}

		if (!angular.equals(dataSourceFilters, self.widget.dataSourceFilter)) {
			self.widget.dataSourceFilter = dataSourceFilters;

			if (self.widget.resetInfinityScroll) {
				self.widget.resetInfinityScroll();
			}
			if(!self.widget.dataSource.getStorageStrategy()){
				updateGroupedData();
			} else if (!widgetIsLazyLoad && self.widget.searchWithPagination){
				self.widget.reload(); // keep current page
			}
		}
	};

	function publishAfterChangeOrderEvent() {
		if (self.widget.afterChangeOrder) {
			self.widget.afterChangeOrder();
		}
	}

	// END: Order by

	// BEGIN: Group and summarization
	$scope.widget.groupedData = [];

	// initialize widget.groups
	updateWidgetGroups();

	if ($scope.widget) {
		$scope.widget.updateGroups = function updateGroups(){
			updateWidgetGroups();
			updateGroupedData();
		};
		$scope.widget.clearSearchList = $scope.widget.clearSearchCriteria = clearSearchCriteria;
		$scope.widget.setSearchList = $scope.widget.setSearchCriteria = setSearchCriteria;
	}

	function updateWidgetGroups() {
		$scope.widget.groups = $scope.widget.fields.filter(function(item) {
			if (item.grouped) item.isVisible = false;
			return item.aclIsVisible && item.grouped;
		}).sort(function(a, b) {
			return a.sequenceInList - b.sequenceInList;
		});

		if ($scope.widget.groups.length) {
			self.updateOrderBy($scope.widget.isLazyLoad());
		}
	}

	function clearSearchCriteria() {
		self.widget.searchCriteria = {};
		self.widget.searchList = "";
	}

	function setSearchCriteria(searchCriteria) {
		self.widget._noReloadFromSearchCriteria = true;

		clearSearchCriteria();
		if (searchCriteria) {
			searchCriteria.forEach(function(field) {
				var fieldValue = field.value;
				if (fieldValue[fieldValue.length - 1] == '%') {
					fieldValue = fieldValue.substring(0, (fieldValue.length - 1));
				}
				if (fieldValue[0] == "%") {
					fieldValue = fieldValue.substring(1, fieldValue.length);
				}
				self.widget.searchCriteria[field.name] = fieldValue;
			});
		}

		searchFromSearchCriteria(self.widget.searchCriteria);
	}

	$scope.$watchCollection('widget.groups', function(newValue, oldValue) {
		if (newValue != oldValue) {
			$scope.widget._gridResize = true; // forces the grid element to recalculate column widths
			self.updateOrderBy();
		}
	});

	$scope.$watch(function() {
		return ZhUtil.clearRow($scope.widget.currentRow);
	}, function() {
		updateGridActionsOnRow($scope.widget.currentRow);
	}, true);

	$scope.$watchCollection('widget.dataSource.data', function() {
		_updateGridActions();
		updateGroupedData();

		$timeout(_redrawElement, 100, false);
	});

	var groups = [];
	var summaryFields = [];
	var summary = {};

	function resetFooterVariables(group) {
		group.footer.count = 0;
		group.footer.min = null;
		group.footer.max = null;

		summaryFields.forEach(function(field) {
			group.footer[field.name] = {};
			group.footer[field.name].sum = 0;
			group.footer[field.name].count = 0;
			group.footer[field.name].min = null;
			group.footer[field.name].max = null;
		});
	}

	function addGroupHeader(group, index, groupOpened) {
		var groupValue = group.lastValue || group.field.groupLabelForEmptyValue;
		var groupHeaderRow = {
			__group: true,
			__groupHeader: true,
			__groupIndex: index,
			__groupLabel: ScreenService.i18n(group.label),
			__groupValue: !group.field.mask ? groupValue : Util.applyMask(groupValue, group.field.mask, maskEngine, ZhUtil),
			__groupOpened: !Util.isUndefinedOrNull(group.field.groupOpened) ? group.field.groupOpened : groupOpened !== false,
			__groupHeaders: []
		};
		// add header for outer groups to the added row
		for (var i = 0; i < index; i++) {
			groupHeaderRow.__groupHeaders.push(groups[i].lastGroupHeaderRow);
		}
		group.lastGroupHeaderRow = groupHeaderRow;
		$scope.widget.groupedData.push(groupHeaderRow);
	}

	function getFieldValues(data, fieldName){
		return data.map(function(row){return row[fieldName];});
	}

	function applyFieldMask(field, value){
		return (!field.mask ? value : Util.applyMask(value, field.mask, maskEngine, ZhUtil));
	}

	function summaryData(group, row, isGroup) {
		var value, lastValue, _group;
		for (var i = 0; i < summaryFields.length; i++) {
			var field = summaryFields[i];
			var expression = field.expression;

			if (isGroup) {
				lastValue = ' (' + (!group.field.mask ? group.lastValue : Util.applyMask(group.lastValue, group.field.mask, maskEngine, ZhUtil)) + ')';
				_group = group.footer;
			} else {
				var mins = [], maxs = [];
				var data = $scope.widget.dataSource.data;

				lastValue = ' (' + ScreenService.i18n('TOTAL') + ')';
				_group = group;

				maxs = mins =  getFieldValues(data, field.name);

				_group[field.name].min = Math.min.apply(null, mins);
				_group[field.name].max = Math.max.apply(null, maxs);
			}

			if (expression == COUNT) {
				row[field.name] = _group[field.name].count;
			} else if (expression == SUM) {
				value = _group[field.name].sum;
				row[field.name] = applyFieldMask(field, value);
			} else if (expression == AVG) {
				value = _group[field.name].sum / _group[field.name].count;
				row[field.name] = applyFieldMask(field, value);
			} else if (expression == MIN) {
				value = _group[field.name].min;
				row[field.name] = applyFieldMask(field, value);
			} else if (expression == MAX) {
				value = _group[field.name].max;
				row[field.name] = applyFieldMask(field, value);
			}
		}
	}

	function addGroupFooters(groupIndex) {
		if (summaryFields.length === 0) {
			return;
		}

		for (var i = groups.length - 1; i >= groupIndex; i--) {
			if (groups[i].footer.count > 0) {
				var groupFooterRow = {
					__group: true,
					__groupFooter: true,
					__groupIndex: i,
					__groupHeaders: [],
					__groupLabel: groups[i].lastGroupHeaderRow.__groupLabel,
					__groupValue: groups[i].lastGroupHeaderRow.__groupValue
				};

				// add header for outer groups
				for (var g = 0; g < i; g++) {
					groupFooterRow.__groupHeaders.push(groups[g].lastGroupHeaderRow);
				}

				summaryData(groups[i], groupFooterRow, true);

				$scope.widget.groupedData.push(groupFooterRow);
			}
		}
	}

	function updateFootersByExpression(data, shouldSumAll) {
		summaryFields.forEach(function(field) {
			if(field.expression.startsWith('@')){
				var fn = $scope.$eval(field.expression.substr(1).slice(0, -1), locals);

				sendArraysToCustomExpressionCallback(fn, data, field.name, shouldSumAll);
			}
		});
	}

	function sendArraysToCustomExpressionCallback(fn, data, fieldName, shouldSumAll) {
		var currentGroup = [];
		var groupName = data[0].__groupLabel;
		var smallGroup = [];
		var smallGroupName = "";
		var smallGroupExists = false;
		var sumAll = 0;

		data.forEach(function(row) {
			if(row.__groupFooter){
				if(row.__groupLabel === groupName){
					row[fieldName] = fn(currentGroup);
					if(shouldSumAll){
						sumAll += row[fieldName];
					}
				} else {
					smallGroup.push(row);
					if(smallGroupExists && row.__groupLabel === smallGroupName){
						sendArraysToCustomExpressionCallback(fn, smallGroup, fieldName, false);
						smallGroupExists = false;
					}
				}
			} else {
				if(row.__groupHeader){
					if(row.__groupLabel === groupName){
						currentGroup = [];
					} else {
						if(smallGroupExists){
							if(row.__groupLabel === smallGroupName){
								smallGroup = [];
							}
							smallGroup.push(row);
						} else {
							smallGroupName = row.__groupLabel;
							smallGroupExists = true;
							smallGroup = [];
							smallGroup.push(row);
						}
					}
				} else {
					if(smallGroupExists){
						smallGroup.push(row);
					}
					currentGroup.push(row);
				}
			}
		});

		if(shouldSumAll){
			data[data.length - 1][fieldName] = sumAll;
		}
	}

	function addSummary() {
		if (summaryFields.length === 0 || groups.length === 0) return;

		var summaryRow = {
			__group: true,
			__groupFooter: true,
			__groupSummary: true
		};

		summaryData(summary, summaryRow);

		$scope.widget.groupedData.push(summaryRow);
	}

	function isFieldNameExpression(field, row){
		return ([SUM, AVG, MAX, MIN].indexOf(field.expression) !== -1) && !isNaN(row[field.name]);
	}

	function calcSummary(row) {
		groups.forEach(function(group) {
			group.footer.count += 1;

			if (summaryFields.length > 0) {
				summaryFields.forEach(function(field) {
					var fieldName = field.name;
					if (!Util.isUndefined(row[fieldName])) {
						group.footer[fieldName].count += 1;
					}
					if (isFieldNameExpression(field, row)) {
						var rowValue = parseFloat(row[fieldName]) || 0;
						var localSummary = group.footer[fieldName];
						localSummary.sum += rowValue;
						localSummary.min = (localSummary.min == null || localSummary.min > rowValue) ? rowValue : localSummary.min;
						localSummary.max = (localSummary.max == null || localSummary.max < rowValue) ? rowValue : localSummary.max;
					}
				});
			}
		});

		summaryFields.forEach(function(field) {
			var fieldName = field.name;
			if (Util.isUndefined(summary[fieldName])) {
				summary[fieldName] = {
					count: 0,
					sum: 0,
					min: null,
					max: null
				};
			}

			if (!Util.isUndefined(row[fieldName])) {
				summary[fieldName].count += 1;
			}

			if (isFieldNameExpression(field, row)) {
				var rowValue = parseFloat(row[fieldName]) || 0;
				var fieldSummary = summary[fieldName];
				fieldSummary.sum += rowValue;
				fieldSummary.min = (fieldSummary.min == null || fieldSummary.min > rowValue) ? rowValue : fieldSummary.min;
				fieldSummary.max = (fieldSummary.max == null || fieldSummary.max < rowValue) ? rowValue : fieldSummary.max;
			}
		});
	}

	function initSummaryFields() {
		summaryFields = $scope.widget.fields.filter(function(field) {
			return !!field.expression && !field.grouped;
		});
		summary = {};
	}

	function initGroups() {
		groups = [];
		$scope.widget.groups.forEach(function(group, index) {
			var newGroup = {
				field: group,
				name: group.name,
				label: group.label,
				lastValue: undefined,
				footer: {}
			};
			resetFooterVariables(newGroup);

			groups.push(newGroup);
		});
	}

	function getOrderedDataByColumns(dataSet, orderByColumns) {
		return orderByColumns.length ? $filter('orderBy')(dataSet, orderByColumns) : dataSet;
	}

	$scope.widget.getUpdatedGroupData = function(data){
		updateGroupedData(data, true);
		return $scope.widget.groupedData;
	};

	function updateGroupedData(data) {
		var widgetDataSource = $scope.widget.dataSource;
		if (Util.isUndefined(widgetDataSource) ||
			(Util.isEmptyOrBlank(widgetDataSource.data) && Util.isEmptyOrBlank(data))) {

			$scope.widget.groupedData = [];
		} else {
			var filteredData = data || widgetDataSource.data;

			if (!widgetDataSource.getStorageStrategy() || !$scope.widget.searchWithPagination || $scope.widget.mode === ViewMode.INLINE_EDIT) {
				filteredData = filteredData.filter(self.search);
			}

			self.updateOrderByColumns($scope.widget);

			// Removes any group header if widget has nothing to group by
			if (Util.isUndefined($scope.widget.groups) || $scope.widget.groups.length === 0) {
				$scope.widget.groupedData = getOrderedDataByColumns(filteredData, self.getOrderByColumns());

				// remove group headers
				$scope.widget.groupedData.forEach(function(row) {
					delete row.__groupHeaders;
				});
			} else {
				$scope.widget.groupedData = [];

				initSummaryFields();
				initGroups();

				var orderedData = getOrderedDataByColumns(filteredData, self.getOrderByColumns());
				var firstField = $scope.widget._filteredFields && $scope.widget._filteredFields[0];
				var totalizerLabel = widgetDataSource.totalizer && widgetDataSource.totalizerLabel;

				orderedData.forEach(function(row, index) {
					if(row.totalizerItem && totalizerLabel && firstField){
						row[firstField.name] = totalizerLabel;
					}

					var groupBreak = false;
					groups.forEach(function(group, index) {
						var headerValue = row[group.name];
						var modelHeaderValue = group.field && DecodeValueFieldService.model(group.field, headerValue);
						var lastValue =  modelHeaderValue || headerValue;
						if (group.lastValue !== lastValue || groupBreak) {
							if (!groupBreak) addGroupFooters(index);
							group.lastValue = lastValue;
							resetFooterVariables(group);

							addGroupHeader(group, index, !$scope.widget.bringGroupsCollapsed);
							groupBreak = true;
						}
					});

					row.__groupHeaders = groups.map(function(group) {
						return group.lastGroupHeaderRow;
					}).filter(function(group){
						return !Util.isUndefined(group);
					});
					$scope.widget.groupedData.push(row);

					calcSummary(row);
				});

				if ($scope.widget.groupedData.length > 0) {
					addGroupFooters(0);

					if($scope.widget.showSummaryTotal !== false){
						addSummary();
					}

					updateFootersByExpression($scope.widget.groupedData, ($scope.widget.showSummaryTotal !== false));
				}
			}
		}

		if (self.widget._updateVisibleRows) {
			self.widget._updateVisibleRows();
		}
	}

	this.toggleGroup = function(row) {
		row.__groupOpened = !row.__groupOpened;

		if (self.widget._updateVisibleRows) {
			self.widget._updateVisibleRows();
			_redrawElement();
		}
	};

	this.getClosedStatus = function(row) {
		var status = false;
		if (row && row.__groupHeaders && row.__groupHeaders.length > 0) {
			status = row.__groupHeaders.some(function(group) {
				return group.__groupOpened === false;
			});
		}
		return status;
	};

	this.getVisibleRowCount = function getVisibleRowCount() {
		var visibleRowsCount = [];
		var widget = $scope.widget;

		if(widget._getVisibilityParams) {
			widget._getVisibilityParams();
			if (widget.groupedData) {
				var dataLength = widget.groupedData.length;
				var rowCount = Math.min(dataLength, widget._visibilityParams.maxVisibleRows);

				visibleRowsCount = Util.range(rowCount);

				self._rows = self.getDataSliceFromRowCount(widget, rowCount);

				if (widget._hiddenRows !== dataLength - rowCount) {
					widget._hiddenRows = dataLength - rowCount;
					widget._calcScrollAreas();
				}
			}
		}

		return visibleRowsCount;
	};

	$scope.widget._updateVisibleRows = Util.buildDebounceMethod(function() {
		if ($scope.widget.groupedData && $scope.widget.groupedData.length > 0) {
			self._rows = self.getDataSliceFromRowCount($scope.widget, self._rows? self._rows.length : 0);

			if (Util.isMobile() && $scope.widget._calcScrollAreas) {
				$scope.widget._calcScrollAreas();
			}

			templateManager.updateTemplate();
		}
	}, 200);

	this.getDataSliceFromRowCount = function getDataSliceFromRowCount(widget, rowCount) {
		var dataSlice = [];
		var data = widget.groupedData;
		if (data === undefined) {
			data = [];
		}
		var topRow = $scope.widget._topRow || 0;
		var lastRow = topRow + rowCount;
		dataSlice = data.slice(topRow, lastRow);
		widget._hideLoadMore = data.length != lastRow;
		return dataSlice;
	};


	/* STRIPE FOOTER */
	this.stripeFooter = function stripeFooter() {
		var stripeWidget;

		if ($scope.widget && $scope.widget.widgets) {
			stripeWidget = $scope.widget.widgets.filter(function(widget) {
				return ~widget.template.indexOf("stripe.html");
			}).shift();
		}

		return stripeWidget;
	};

	var _widget = self.widget;
	$scope.$on('$destroy', function() {
		delete _widget.toggle;
		delete _widget._updateVisibleRows;
		delete _widget.updateGridActions;
		delete _widget.clearSearchList;
		delete _widget.clearSearchCriteria;
		delete _widget.setSearchList;
		delete _widget.setSearchCriteria;
		delete _widget.updateGroups;
		delete _widget.groupedData;

		uiDeselectAllDeregister();
		uiAllowMultiSelectDeregister();
		uiDisableMultiSelectDeregister();
	});
}